﻿// Import: Jan Conze / PBS2H23ACO mit Unterstützung durch Radu Catalin Calcan

using Core;
using Projekt_Calcan_Conze_Import.DTOs;
using Projekt_Calcan_Conze_Import.Models;
using System.Net.Mail;

namespace Projekt_Calcan_Conze_Import;

internal static class ImportUsers
{
    public static (List<User> User, List<string> Protocol) FromCsv(string filePath)
    {
        var (userDtos, protocol) = ReadUserDtos(filePath);
        var users = ParseUsers(userDtos, protocol);
        return (users, protocol);
    }

    private static (List<UserDto> UserDtos, List<string> Protocol) ReadUserDtos(string filePath)
    {
        List<string> protocol = [];
        List<UserDto> userDtos = [];
        List<string> lines;

        try
        {
            lines =
                File.ReadAllLines(filePath)
                    .Where(line => !string.IsNullOrWhiteSpace(line)) // remove empty lines
                    .ToList();
        }
        catch (Exception)
        {
            Console.WriteLine("The file does not exist or is currently used by another program.");
            return (userDtos, protocol);
        }

        UserDto? currentUser = null;

        foreach (string line in lines)
        {
            var lineParts =
                line.Split(Constants.Separator)
                    .Select(linePart => linePart.Trim())
                    .Where(linePart => !string.IsNullOrWhiteSpace(linePart)) // remove empty parts
                    .ToList(); // remove whitespace

            string title = lineParts[0];

            if (title
                is Constants.FemaleAttributeIdentifier
                or Constants.MaleAttributeIdentifier
                or Constants.DiverseAttributeIdentifier)
            {
                if (currentUser is not null)
                {
                    userDtos.Add(currentUser);
                }

                currentUser = new UserDto([new UserAttributeDto(lineParts, line)]);
            }
            else if (currentUser is not null)
            {
                currentUser.Attributes.Add(new UserAttributeDto(lineParts, line));
            }
            else
            {
                protocol.Add(line);
            }
        }

        if (currentUser is not null)
        {
            userDtos.Add(currentUser);
        }

        return (userDtos, protocol);
    }

    private static List<User> ParseUsers(List<UserDto> userDtos, List<string> protocol)
    {
        List<User> users = [];

        foreach (var userDto in userDtos)
        {
            var user = ParseUser(userDto);

            if (user is not null)
            {
                users.Add(user);
            }
            else
            {
                protocol.AddRange(userDto.Attributes.Select(attribute => attribute.OriginalCsvLine));
            }
        }

        return users;
    }

    private static User? ParseUser(UserDto user)
    {
        var firstAttribute = user.Attributes[0];

        if (firstAttribute.Parts.Count != 4)
        {
            return null;
        }

        string title = firstAttribute.Parts[0];
        string fistName = firstAttribute.Parts[1];
        string lastName = firstAttribute.Parts[2];

        DateOnly? dateOfBirth =
            DateOnly.TryParseExact(firstAttribute.Parts[3], Constants.DateOfBirthFormat,
                                   out var date)
                ? date
                : null;

        if (dateOfBirth is null)
        {
            return null;
        }

        string? email = null;
        Address? address = null;
        List<PhoneNumber> phoneNumbers = [];

        foreach (var attribute in user.Attributes.Skip(1))
        {
            var attributeParts = attribute.Parts;
            string identifier = attributeParts[0];

            switch (identifier)
            {
                case Constants.AddressAttributeIdentifier:
                    if (attributeParts.Count == 4)
                    {
                        string postalCode = attributeParts[2];

                        if (postalCode.Length == 5)
                        {
                            address =
                                new Address(
                                    streetAndHouseNumber: attributeParts[1],
                                    postalCode: postalCode,
                                    city: attributeParts[3]);
                        }
                        else
                        {
                            return null;
                        }
                    }

                    break;

                case Constants.EmailAttributeIdentifier:
                    if (attributeParts.Count == 2)
                    {
                        try
                        {
                            email = attributeParts[1];

                            _ = new MailAddress(
                                email); // validate the format of the email address
                        }
                        catch (Exception)
                        {
                            return null;
                        }
                    }
                    else
                    {
                        return null;
                    }

                    break;

                case Constants.PhoneNumberAttributeIdentifier:
                    if (attributeParts.Count == 3)
                    {
                        string areaCode = attributeParts[1];
                        string number = attributeParts[2];

                        if (areaCode.Length is >= 3 and <= 5
                            && number.Length is >= 4 and <= 10)
                        {
                            phoneNumbers.Add(new PhoneNumber(areaCode, number));
                        }
                        else
                        {
                            return null;
                        }
                    }
                    else
                    {
                        return null;
                    }

                    break;

                default: return null;
            }
        }

        return new User(
            title: title,
            firstName: fistName,
            lastName: lastName,
            dateOfBirth: dateOfBirth.Value,
            email: email,
            address: address,
            phoneNumbers: phoneNumbers);
    }
}