﻿// Import: Jan Conze / PBS2H23ACO mit Unterstützung durch Radu Catalin Calcan

using Core;
using System.Data;

namespace Projekt_Calcan_Conze_Import.Repositories;

using MySql.Data.MySqlClient;
using Projekt_Calcan_Conze_Import.DTOs;
using Projekt_Calcan_Conze_Import.Models;

internal static class Database
{

    public static async Task<List<UserIdentifierDto>> GetUserIdentifiers(string clientNumber)
    {
        var users = new List<UserIdentifierDto>();

        try
        {
            // use "using" to automatically close the connection when done
            await using var dbConnection = new MySqlConnection(Constants.ConnectionString);

            // open a connection
            await dbConnection.OpenAsync();

            var usersCommand =
                new MySqlCommand
                {
                    Connection = dbConnection,
                    CommandText =
                        $"""
                         SELECT *
                         FROM user
                         INNER JOIN client ON user.clientId = client.id
                         INNER JOIN gender ON user.genderId = gender.id
                         WHERE client.clientno = @{nameof(clientNumber)}
                         """
                };
            usersCommand.Parameters.AddWithValue($"@{nameof(clientNumber)}", clientNumber);

            // execute the command and read the results
            await using var sqlReader = await usersCommand.ExecuteReaderAsync();

            while (await sqlReader.ReadAsync())
            {
                string title = sqlReader.GetString("description");
                string firstName = sqlReader.GetString("firstname");
                string lastName = sqlReader.GetString("lastname");

                users.Add(
                    new UserIdentifierDto(
                        title: title,
                        firstName: firstName,
                        lastName: lastName));
            }
        }
        catch (Exception ex)
        {
            Console.WriteLine("An error occurred while reading the database: " + ex.Message);
        }

        return users;
    }

    public static async Task CreateUsers(List<User> usersToCreate, string clientNumber)
    {
        try
        {
            await using var dbConnection = new MySqlConnection(Constants.ConnectionString);
            await dbConnection.OpenAsync();
            var transaction = await dbConnection.BeginTransactionAsync();

            try
            {
                var getClientIdCommand =
                    new MySqlCommand
                    {
                        Connection = dbConnection,
                        Transaction = transaction,
                        CommandText = "SELECT id FROM client WHERE clientno = @clientno"
                    };
                getClientIdCommand.Parameters.AddWithValue("@clientno", clientNumber);
                object? clientIdResult = await getClientIdCommand.ExecuteScalarAsync();
                long clientId;

                if (clientIdResult is int cId)
                {
                    clientId = cId;
                }
                else
                {
                    throw new Exception("Client not found");
                }

                foreach (var user in usersToCreate)
                {
                    var getGenderCommand =
                        new MySqlCommand
                        {
                            Connection = dbConnection,
                            Transaction = transaction,
                            CommandText = "SELECT id FROM gender WHERE gender.description = @title "
                        };
                    getGenderCommand.Parameters.AddWithValue("@title", user.Title);

                    await using var genderReader = await getGenderCommand.ExecuteReaderAsync();

                    long genderId;

                    if (await genderReader.ReadAsync())
                    {
                        genderId = genderReader.GetByte("id");
                        await genderReader.CloseAsync();
                    }
                    else
                    {
                        await genderReader.CloseAsync();
                        genderId =
                            await GetNextIdForTable(
                                table: "gender",
                                idColumn: "id",
                                idType: typeof(byte),
                                dbConnection,
                                transaction);

                        var insertGenderCommand =
                            new MySqlCommand
                            {
                                Connection = dbConnection,
                                Transaction = transaction,
                                CommandText =
                                     """
                                     INSERT INTO gender (id, description)
                                     VALUES (@id, @title) 
                                     """
                            };
                        insertGenderCommand.Parameters.AddWithValue("@id", genderId);
                        insertGenderCommand.Parameters.AddWithValue("@title", user.Title);
                        insertGenderCommand.ExecuteNonQuery();
                    }

                    long userId =
                        await GetNextIdForTable(
                            table: "user",
                            idColumn: "id",
                            idType: typeof(int),
                            dbConnection,
                            transaction);
                    var userCommand =
                        new MySqlCommand
                        {
                            Connection = dbConnection,
                            Transaction = transaction,
                            CommandText =
                                 """
                                 INSERT INTO user (id, clientId, genderId, firstname, lastname, birthdate)
                                 VALUES (@id, @clientId, @genderId, @firstname, @lastname, @dateofbirth)
                                 """
                        };
                    userCommand.Parameters.AddWithValue("@id", userId);
                    userCommand.Parameters.AddWithValue("@clientId", clientId);
                    userCommand.Parameters.AddWithValue("@genderId", genderId);
                    userCommand.Parameters.AddWithValue("@firstname", user.FirstName);
                    userCommand.Parameters.AddWithValue("@lastname", user.LastName);
                    userCommand.Parameters.AddWithValue("@dateofbirth", user.DateOfBirth.ToString("yyyy-MM-dd"));

                    await userCommand.ExecuteNonQueryAsync();


                    if (user.Email is not null)
                    {
                        long emailId =
                            await GetNextIdForTable(
                                table: "email",
                                idColumn: "id",
                                idType: typeof(int),
                                dbConnection,
                                transaction);
                        var emailCommand =
                            new MySqlCommand
                            {
                                Connection = dbConnection,
                                Transaction = transaction,
                                CommandText =
                                     """
                                     INSERT INTO email (id, userId, email)
                                     VALUES (@id, @userId, @email)
                                     """
                            };
                        emailCommand.Parameters.AddWithValue("@id", emailId);
                        emailCommand.Parameters.AddWithValue("@userId", userId);
                        emailCommand.Parameters.AddWithValue("@email", user.Email);

                        await emailCommand.ExecuteNonQueryAsync();
                    }

                    if (user.Address is not null)
                    {
                        long addressId =
                            await GetNextIdForTable(
                                table: "address",
                                idColumn: "id",
                                idType: typeof(int),
                                dbConnection,
                                transaction);
                        var addressCommand =
                            new MySqlCommand
                            {
                                Connection = dbConnection,
                                Transaction = transaction,
                                CommandText =
                                     """
                                     INSERT INTO address (id, userId, street, postalcode, city)
                                     VALUES (@id, @userId, @street, @postalcode, @city)
                                     """
                            };
                        addressCommand.Parameters.AddWithValue("@id", addressId);
                        addressCommand.Parameters.AddWithValue("@userId", userId);
                        addressCommand.Parameters.AddWithValue("@street", user.Address.StreetAndHouseNumber);
                        addressCommand.Parameters.AddWithValue("@postalcode", user.Address.PostalCode);
                        addressCommand.Parameters.AddWithValue("@city", user.Address.City);

                        await addressCommand.ExecuteNonQueryAsync();
                    }

                    foreach (var phoneNumber in user.PhoneNumbers)
                    {
                        long phoneId =
                            await GetNextIdForTable(
                                table: "phone",
                                idColumn: "id",
                                idType: typeof(int),
                                dbConnection,
                                transaction);
                        var phoneNumberCommand =
                            new MySqlCommand
                            {
                                Connection = dbConnection,
                                Transaction = transaction,
                                CommandText =
                                     """
                                     INSERT INTO phone (id, userId, phoneprefix, phonenumber)
                                     VALUES (@id, @userId, @areacode, @number)
                                     """
                            };
                        phoneNumberCommand.Parameters.AddWithValue("@id", phoneId);
                        phoneNumberCommand.Parameters.AddWithValue("@userId", userId);
                        phoneNumberCommand.Parameters.AddWithValue("@areacode", phoneNumber.AreaCode);
                        phoneNumberCommand.Parameters.AddWithValue("@number", phoneNumber.Number);

                        await phoneNumberCommand.ExecuteNonQueryAsync();
                    }
                }
            }
            catch (Exception)
            {
                await transaction.RollbackAsync();
                throw;
            }

            await transaction.CommitAsync();
        }
        catch (Exception ex)
        {
            Console.WriteLine("An error occurred while writing to the database: " + ex.Message);
        }
    }

    private static async Task<long> GetNextIdForTable(
        string table,
        string idColumn,
        Type idType,
        MySqlConnection connection,
        MySqlTransaction transaction)
    {
        var getMaxIdCommand =
            new MySqlCommand
            {
                Connection = connection,
                Transaction = transaction,
                CommandText = $"SELECT MAX({idColumn}) FROM {table}"
            };
        await using var maxIdReader = await getMaxIdCommand.ExecuteReaderAsync();
        return await maxIdReader.ReadAsync()
            ? idType switch
            {
                { } t when t == typeof(byte) => maxIdReader.GetByte(0) + 1,
                { } t when t == typeof(short) => maxIdReader.GetInt16(0) + 1,
                { } t when t == typeof(int) => maxIdReader.GetInt32(0) + 1,
                { } t when t == typeof(long) => maxIdReader.GetInt64(0) + 1,
                _ => throw new ArgumentException("Unsupported id type")
            }
            : 1;
    }
}